﻿################################################################################################
# This script is not supported under any Microsoft standard support program or service.
# This script is provided AS IS without warranty of any kind.
# Microsoft further disclaims all implied warranties including, without limitation, any implied
# warranties of merchantability or of fitness for a particular purpose. The entire risk arising
# out of the use or performance of the sample script and documentation remains with you. In no
# event shall Microsoft, its authors, or anyone else involved in the creation, production, or
# delivery of the scripts be liable for any damages whatsoever (including, without limitation,
# damages for loss of business profits, business interruption, loss of business information,
# or other pecuniary loss) arising out of the use of or inability to use the sample script or
# documentation, even if Microsoft has been advised of the possibility of such damages.
################################################################################################
<#
	.SYNOPSIS
	This script sets up pre-requisites for running DSC in order to cover STIG gaps.

	Additionally, this policy sets the Data Execution Prevention policy to OptOut.
#>

[CmdletBinding()]
param ()

# Test Registry Item Function
function Test-RegItem {
    [CmdletBinding()]
    param (
        [Parameter(Mandatory = $true)]
        [String]
        $Path,

        [Parameter(Mandatory = $true)]
        [String]
        $Value
    )

    try {
        Get-ItemProperty -Path $Path -Name $Value -ErrorAction Stop | Out-Null
        return $true
    }
    catch {
        return $false
    }
}

# Ensure that the proper DSC modules are installed
$destination = "$env:ProgramFiles\WindowsPowerShell\Modules"
$secmodule = Get-Item -Path "$PSScriptRoot\Modules\SecurityPolicyDsc"
$secpresent = Get-InstalledModule | Where-Object -FilterScript {$_.Name -eq 'SecurityPolicyDsc'}

if ($null -eq $secpresent -or $secpresent.Version -lt '2.10.0.0') {
    try {
        Copy-Item -Path $secmodule -Destination $destination -Recurse -Force
    }
    catch {
        Write-Warning $Error[0]
    }
}
else {
    Write-Host "SecurityPolicyDsc module $($secpresent.Version.ToString()) is already installed."
}

# Set up WinRM for DSC
Enable-PSRemoting -SkipNetworkProfileCheck

# Disable Optional Feature Windows PowerShell V2 Root
Disable-WindowsOptionalFeature -Online -FeatureName MicrosoftWindowsPowerShellV2Root

# Set HKCU Values in all user profiles
[array]$users = (Get-ChildItem -Path 'C:\Users' | Where-Object -FilterScript {$_.Name -notlike 'public' -and $_.Name -notlike $env:Username}).Name
$users += 'Default'

$entries = @(
    @{
		Key       = 'Software\Microsoft\Windows\CurrentVersion\WinTrust\Trust Providers\Software Publishing'
		ValueName = 'State'
		ValueData = '146432'
		ValueType = 'DWORD'
	}
)

New-PSDrive -PSProvider Registry -Name HKU -Root HKEY_USERS
foreach ($user in $users) {
    $dat = "C:\Users\$user\NTUSER.DAT"
    $tempLocation = "HKU\$user"

    try {
        reg load $tempLocation $dat

        foreach ($entry in $entries) {
            if ((Test-RegItem -Path "HKU:\$user\$($entry.Key)" -Value $entry.ValueName) -eq $false) {
                New-ItemProperty -Path "HKU:\$user\$($entry.Key)" -Name $entry.ValueName -PropertyType $entry.ValueType -Value $entry.ValueData
            }
            elseif ((Get-ItemProperty -Path "HKU:\$user\$($entry.Key)" | Select-Object -ExpandProperty $entry.ValueName) -ne $entry.ValueData) {
                Set-ItemProperty -Path "HKU:\$user\$($entry.Key)" -Name $entry.ValueName -Value $entry.ValueData
            }
        }

        [gc]::Collect()
        reg unload $tempLocation
    }
    catch {
        Write-Warning -Message "Error modifying default user profile for $user"
        Write-Warning -Message $Error[0]
    }
}

Remove-PSDrive -Name HKU

# Determines bitlocker encrypted volumes
$encryptedVolumes = (Get-BitLockerVolume | Where-Object {$_.ProtectionStatus -eq 'On'}).MountPoint

<# Enables DEP. If there are bitlocker encrypted volumes, bitlocker is temporarily suspended for this operation
Data Execution Prevention (DEP) must be configured to at least OptOut
V-220726 Windows 10
V-253283 Windows 11
#>
if ($encryptedVolumes.Count -gt 0) {
    foreach ($volume in $encryptedVolumes) {
        Suspend-BitLocker -MountPoint $volume
    }

    Start-Process -Wait -FilePath 'C:\Windows\System32\bcdedit.exe' -ArgumentList '/set "{current}" nx OptOut'

    foreach ($volume in $encryptedVolumes) {
        Resume-BitLocker -MountPoint $volume
    }
}
else {
    Start-Process -Wait -FilePath 'C:\Windows\System32\bcdedit.exe' -ArgumentList '/set "{current}" nx OptOut'
}
